@@ -1,15 +1,18 @@
 """Tests generating meshes."""
 import numpy as np
+import warnings
 import pytest
 
 import tidy3d as td
 from tidy3d.constants import fp_eps
 
 from tidy3d.components.grid.mesher import GradedMesher
+from ..utils import assert_log_level
 
 np.random.seed(4)
 
 MESHER = GradedMesher()
+np.random.seed(4)
 
 
 def validate_dl_multiple_interval(
@@ -428,6 +431,162 @@ def test_mesh_high_index_background_override():
     assert np.isclose(sizes[0], WAVELENGTH / 10)
 
 
+def test_mesh_direct_override():
+    """test td.MeshOverrideStructure"""
+
+    # default override takes effect along one axis
+    for axis in range(3):
+        dl = [None] * 3
+        dl[axis] = 0.05
+
+        override_fine = td.MeshOverrideStructure(
+            geometry=td.Box(size=(1, 1, 1)),
+            dl=dl,
+        )
+
+        sim = td.Simulation(
+            size=(3, 3, 3),
+            grid_spec=td.GridSpec.auto(
+                wavelength=WAVELENGTH,
+                override_structures=[override_fine],
+            ),
+            run_time=1e-13,
+            structures=[BOX1],
+        )
+        assert np.all(sim.grid.sizes.to_list[(axis + 1) % 3] > 0.09)
+        assert np.all(sim.grid.sizes.to_list[(axis + 2) % 3] > 0.09)
+        sizes = sim.grid.sizes.to_list[axis]
+        assert np.isclose(sizes[len(sizes) // 2], 0.05)
+
+    # default override takes effect along two axes
+    for axis in range(3):
+        dl = [0.05] * 3
+        dl[axis] = None
+
+        override_fine = td.MeshOverrideStructure(
+            geometry=td.Box(size=(1, 1, 1)),
+            dl=dl,
+        )
+
+        sim = td.Simulation(
+            size=(3, 3, 3),
+            grid_spec=td.GridSpec.auto(
+                wavelength=WAVELENGTH,
+                override_structures=[override_fine],
+            ),
+            run_time=1e-13,
+            structures=[BOX1],
+        )
+        assert np.all(sim.grid.sizes.to_list[axis] > 0.09)
+        sizes = sim.grid.sizes.to_list[(axis + 1) % 3]
+        assert np.isclose(sizes[len(sizes) // 2], 0.05)
+        sizes = sim.grid.sizes.to_list[(axis + 2) % 3]
+        assert np.isclose(sizes[len(sizes) // 2], 0.05)
+
+    # Over all three axes
+    override_fine = td.MeshOverrideStructure(
+        geometry=td.Box(size=(1, 1, 1)),
+        dl=[0.05] * 3,
+    )
+
+    sim = td.Simulation(
+        size=(3, 3, 3),
+        grid_spec=td.GridSpec.auto(
+            wavelength=WAVELENGTH,
+            override_structures=[override_fine],
+        ),
+        run_time=1e-13,
+        structures=[BOX1],
+    )
+    for axis in range(3):
+        sizes = sim.grid.sizes.to_list[axis]
+        assert np.isclose(sizes[len(sizes) // 2], 0.05)
+
+    # default override has no effect when coarser than enclosing structure
+    override_coarse = override_fine.copy(update={"dl": [0.2] * 3})
+    sim = td.Simulation(
+        size=(3, 3, 3),
+        grid_spec=td.GridSpec.auto(
+            wavelength=WAVELENGTH,
+            override_structures=[override_coarse],
+        ),
+        run_time=1e-13,
+        structures=[BOX1],
+    )
+    for axis in range(3):
+        sizes = sim.grid.sizes.to_list[axis]
+        assert sizes[len(sizes) // 2] < 0.1
+
+    # however, when enforced, override takes effect again
+    override_coarse_enforce = override_coarse.copy(update={"enforce": True})
+    sim = td.Simulation(
+        size=(3, 3, 3),
+        grid_spec=td.GridSpec.auto(
+            wavelength=WAVELENGTH,
+            override_structures=[override_coarse_enforce],
+        ),
+        run_time=1e-13,
+        structures=[BOX1],
+    )
+    for axis in range(3):
+        sizes = sim.grid.sizes.to_list[axis]
+        assert sizes[len(sizes) // 2] > 0.15
+
+
+def test_mesh_multiple_direct_override_and_global_min():
+    """Test multiple td.MeshOverrideStructure objects.
+    Let's consider three override structures: enforced + default + enforced;
+    The two enforced structure overlap.
+
+    We also test dl_min here.
+    """
+
+    override_enforce1 = td.MeshOverrideStructure(
+        geometry=td.Box(center=(0, -1, 1), size=(0.4, 0.4, 3)),
+        dl=[None, None, 0.13],
+        enforce=True,
+    )
+
+    override_enforce2 = td.MeshOverrideStructure(
+        geometry=td.Box(center=(0, -1, 0), size=(0.3, 0.3, 1)), dl=[0.15] * 3, enforce=True
+    )
+
+    override_default = td.MeshOverrideStructure(
+        geometry=td.Box(center=(0, 0, 0), size=(1.5, 1.5, 1.5)),
+        dl=[0.05] * 3,
+    )
+
+    sim = td.Simulation(
+        size=(3, 3, 3),
+        grid_spec=td.GridSpec.auto(
+            wavelength=WAVELENGTH,
+            override_structures=[override_enforce1, override_default, override_enforce2],
+        ),
+        run_time=1e-13,
+        structures=[BOX1],
+    )
+    sizes = sim.grid.sizes.to_list[2]
+    assert max(sizes) > 0.14
+    assert min(sizes) <= 0.05
+    assert sizes[-1] > 0.12
+
+    # now let's set a dl_min
+    sim = td.Simulation(
+        size=(3, 3, 3),
+        grid_spec=td.GridSpec.auto(
+            wavelength=WAVELENGTH,
+            override_structures=[override_enforce1, override_default, override_enforce2],
+            dl_min=0.1,
+        ),
+        run_time=1e-13,
+        structures=[BOX1],
+    )
+    sizes = sim.grid.sizes.to_list[2]
+    assert max(sizes) > 0.14
+    assert min(sizes) >= 0.08
+    assert sizes[-1] > 0.12
+
+
 def test_mesh_gold_slab():
     # Test meshing of a slab with large negative permittivity
     gold_step = WAVELENGTH / 10 / np.sqrt(np.abs(GOLD.eps_model(td.C_0 / WAVELENGTH).real))
@@ -490,3 +649,16 @@ def test_mesher_timeout():
     )
 
     grid = sim.grid
+
+
+def test_shapely_strtree_warnings(caplog):
+
+    with warnings.catch_warnings():
+        warnings.simplefilter("error")
+        m = GradedMesher().parse_structures(
+            axis=2,
+            structures=[BOX1, BOX2],
+            wavelength=1.0,
+            min_steps_per_wvl=6,
+            dl_min=1.0,
+        )
