@@ -1,26 +1,31 @@
 """Defines electric current sources for injecting light into simulation."""
 
 from abc import ABC, abstractmethod
-from typing import Union, Tuple
-import logging
+from typing import Union, Tuple, Optional
 
 from typing_extensions import Literal
 import pydantic
 import numpy as np
 
-from .base import Tidy3dBaseModel, cached_property
+from .base import Tidy3dBaseModel, cached_property, DATA_ARRAY_TAG
 from .types import Direction, Polarization, Ax, FreqBound, ArrayLike, Axis, Bound
-from .validators import assert_plane, validate_name_str
+from .validators import assert_plane, validate_name_str, get_value
+from .data.dataset import FieldDataset
 from .geometry import Box
 from .mode import ModeSpec
 from .viz import add_ax_if_none, PlotParams, plot_params_source
 from .viz import ARROW_COLOR_SOURCE, ARROW_ALPHA, ARROW_COLOR_POLARIZATION
 from ..constants import RADIAN, HERTZ, MICROMETER, GLANCING_CUTOFF
 from ..constants import inf  # pylint:disable=unused-import
-from ..log import SetupError
+from ..log import SetupError, log
 
 # in spectrum computation, discard amplitudes with relative magnitude smaller than cutoff
 DFT_CUTOFF = 1e-8
+# when checking if custom data spans the source plane, allow for a small tolerance
+# due to numerical precision
+DATA_SPAN_TOL = 1e-8
+# width of Chebyshev grid used for broadband sources (in units of pulse width)
+CHEB_GRID_WIDTH = 1.5
 
 
 class SourceTime(ABC, Tidy3dBaseModel):
@@ -312,7 +317,7 @@ class Source(Box, ABC):
         z: float = None,
         ax: Ax = None,
         sim_bounds: Bound = None,
-        **patch_kwargs
+        **patch_kwargs,
     ) -> Ax:
 
         # call the `Source.plot()` function first.
@@ -394,26 +399,18 @@ class PointDipole(CurrentSource):
     >>> pt_dipole = PointDipole(center=(1,2,3), source_time=pulse, polarization='Ex')
     """
 
-    size: Tuple[Literal[0], Literal[0], Literal[0]] = (0, 0, 0)
+    size: Tuple[Literal[0], Literal[0], Literal[0]] = pydantic.Field(
+        (0, 0, 0),
+        title="Size",
+        description="Size in x, y, and z directions, constrained to ``(0, 0, 0)``.",
+        units=MICROMETER,
+    )
 
 
 class FieldSource(Source, ABC):
     """A Source defined by the desired E and/or H fields."""
 
 
-""" TODO: Custom currents """
-
-
-class CustomSource(Source, ABC):
-    """Implements custom current components specified by data."""
-
-    data: ArrayLike[float, 2]
-
-
-class CustomFieldSource(FieldSource, CustomSource):
-    """Implements custom E, H fields specified by data."""
-
-
 """ Field Sources can be defined either on a (1) surface or (2) volume. Defines injection_axis """
 
 
@@ -463,6 +460,155 @@ class DirectionalSource(FieldSource, ABC):
         return None
 
 
+class BroadbandSource(Source, ABC):
+    """A source with frequency dependent field distributions."""
+
+    num_freqs: int = pydantic.Field(
+        1,
+        title="Number of Frequency Points",
+        description="Number of points used to approximate the frequency dependence of injected "
+        "field. A Chebyshev interpolation is used, thus, only a small number of points, i.e., less "
+        "than 20, is typically sufficient to obtain converged results.",
+        ge=1,
+        le=99,
+    )
+
+    @cached_property
+    def frequency_grid(self) -> np.ndarray:
+        """A Chebyshev grid used to approximate frequency dependence."""
+        freq_min, freq_max = self.source_time.frequency_range(num_fwidth=CHEB_GRID_WIDTH)
+        freq_avg = 0.5 * (freq_min + freq_max)
+        freq_diff = 0.5 * (freq_max - freq_min)
+        uni_points = (2 * np.arange(self.num_freqs) + 1) / (2 * self.num_freqs)
+        cheb_points = np.cos(np.pi * np.flip(uni_points))
+        return freq_avg + freq_diff * cheb_points
+
+    @pydantic.validator("num_freqs", always=True, allow_reuse=True)
+    def _warn_if_large_number_of_freqs(cls, val):
+        """Warn if a large number of frequency points is requested."""
+
+        if val is None:
+            return val
+
+        if val >= 20:
+            log.warning(
+                f"A large number ({val}) of frequency points is used in a broadband source. "
+                "This can slow down simulation time and is only needed if the mode fields are "
+                "expected to have a very sharp frequency dependence. 'num_freqs' < 20 is "
+                "sufficient in most cases."
+            )
+
+        return val
+
+
+""" Source current profiles determined by user-supplied data on a plane."""
+
+
+class CustomFieldSource(FieldSource, PlanarSource):
+    """Implements a source corresponding to an input dataset containing ``E`` and ``H`` fields.
+    For the injection to work as expected, the fields must decay by the edges of the source plane,
+    or the source plane must span the entire simulation domain and the fields must match the
+    simulation boundary conditions. The equivalent source currents are fully defined by the field
+    components tangential to the source plane. The normal components (e.g. ``Ez`` and ``Hz``) can be
+    provided but will have no effect on the results, in accordance with the equivalence principle.
+    At least one of the tangential components has to be defined. For example, for a ``z``-normal
+    source, at least one of ``Ex``, ``Ey``, ``Hx``, and ``Hy`` has to be present in the provided
+    dataset. The coordinates of all provided fields are assumed to be relative to the source
+    center. Each provided field component must also span the size of the source.
+
+    Note
+    ----
+        If only the ``E`` or only the ``H`` fields are provided, the source will not be directional,
+        but will inject equal power in both directions instead.
+
+    Example
+    -------
+    >>> from tidy3d import ScalarFieldDataArray
+    >>> pulse = GaussianPulse(freq0=200e12, fwidth=20e12)
+    >>> x = np.linspace(-1, 1, 101)
+    >>> y = np.linspace(-1, 1, 101)
+    >>> z = np.array([0])
+    >>> f = [2e14]
+    >>> coords = dict(x=x, y=y, z=z, f=f)
+    >>> scalar_field = ScalarFieldDataArray(np.ones((101, 101, 1, 1)), coords=coords)
+    >>> dataset = FieldDataset(Ex=scalar_field)
+    >>> custom_source = CustomFieldSource(
+    ...     center=(1, 1, 1),
+    ...     size=(2, 2, 0),
+    ...     source_time=pulse,
+    ...     field_dataset=dataset)
+
+    """
+
+    field_dataset: Optional[FieldDataset] = pydantic.Field(
+        ...,
+        title="Field Dataset",
+        description=":class:`.FieldDataset` containing the desired frequency-domain "
+        "fields patterns to inject. At least one tangetial field component must be specified.",
+    )
+
+    @pydantic.validator("field_dataset", pre=True, always=True)
+    def _warn_if_none(cls, val: FieldDataset) -> FieldDataset:
+        """Warn if the DataArrays fail to load."""
+        if isinstance(val, dict):
+            if DATA_ARRAY_TAG in [v for v in val.values() if isinstance(v, str)]:
+                log.warning("Loading 'field_dataset' without data.")
+                return None
+        return val
+
+    @pydantic.validator("field_dataset", always=True)
+    def _single_frequency_in_range(cls, val: FieldDataset, values: dict) -> FieldDataset:
+        """Assert only one frequency supplied and it's in source time range."""
+        if val is None:
+            return val
+        source_time = get_value(key="source_time", values=values)
+        fmin, fmax = source_time.frequency_range()
+        for name, scalar_field in val.field_components.items():
+            freqs = scalar_field.f
+            if len(freqs) != 1:
+                raise SetupError(
+                    f"'field_dataset.{name}' must have a single frequency, "
+                    f"contains {len(freqs)} frequencies."
+                )
+            freq = float(freqs[0])
+            if (freq < fmin) or (freq > fmax):
+                raise SetupError(
+                    f"'field_dataset.{name}' contains frequency: {freq:.2e} Hz, which is outside "
+                    f"of the 'source_time' frequency range [{fmin:.2e}-{fmax:.2e}] Hz."
+                )
+        return val
+
+    @pydantic.validator("field_dataset", always=True)
+    def _tangential_component_defined(cls, val: FieldDataset, values: dict) -> FieldDataset:
+        """Assert that at least one tangential field component is provided."""
+        if val is None:
+            return val
+        size = get_value(key="size", values=values)
+        normal_axis = size.index(0.0)
+        _, (cmp1, cmp2) = cls.pop_axis("xyz", axis=normal_axis)
+        for field in "EH":
+            for cmp_name in (cmp1, cmp2):
+                tangential_field = field + cmp_name
+                if tangential_field in val.field_components:
+                    return val
+        raise SetupError("No tangential field found in the suppled 'field_dataset'.")
+
+    @pydantic.validator("field_dataset", always=True)
+    def _tangential_fields_span_source(cls, val: FieldDataset, values: dict) -> FieldDataset:
+        """Assert that provided data spans source bounds in the frame with the source center as the
+        origin."""
+        if val is None:
+            return val
+        size = get_value(key="size", values=values)
+        for name, field in val.field_components.items():
+            for dim, dim_name in enumerate("xyz"):
+                in_bounds_min = np.amin(field.coords[dim_name]) <= -size[dim] / 2 + DATA_SPAN_TOL
+                in_bounds_max = np.amax(field.coords[dim_name]) >= size[dim] / 2 - DATA_SPAN_TOL
+                if not (in_bounds_min and in_bounds_max):
+                    raise SetupError(f"Data for field {name} does not span the source plane.")
+        return val
+
+
 """ Source current profiles defined by (1) angle or (2) desired mode. Sets theta and phi angles."""
 
 
@@ -506,7 +652,7 @@ class AngledFieldSource(DirectionalSource, ABC):
     def glancing_incidence(cls, val):
         """Warn if close to glancing incidence."""
         if np.abs(np.pi / 2 - val) < GLANCING_CUTOFF:
-            logging.warning(
+            log.warning(
                 "Angled source propagation axis close to glancing angle. "
                 "For best results, switch the injection axis."
             )
@@ -538,7 +684,7 @@ class AngledFieldSource(DirectionalSource, ABC):
         return self.rotate_points(pol_vector_p, propagation_dir, angle=self.pol_angle)
 
 
-class ModeSource(DirectionalSource, PlanarSource):
+class ModeSource(DirectionalSource, PlanarSource, BroadbandSource):
     """Injects current source to excite modal profile on finite extent plane.
 
     Example
@@ -592,7 +738,7 @@ class ModeSource(DirectionalSource, PlanarSource):
 
 
 class PlaneWave(AngledFieldSource, PlanarSource):
-    """Uniform current distribution on an infinite extent plane.  One element of size must be zero.
+    """Uniform current distribution on an infinite extent plane. One element of size must be zero.
 
     Example
     -------
@@ -601,7 +747,7 @@ class PlaneWave(AngledFieldSource, PlanarSource):
     """
 
 
-class GaussianBeam(AngledFieldSource, PlanarSource):
+class GaussianBeam(AngledFieldSource, PlanarSource, BroadbandSource):
     """Guassian distribution on finite extent plane.
 
     Example
@@ -630,7 +776,7 @@ class GaussianBeam(AngledFieldSource, PlanarSource):
     )
 
 
-class AstigmaticGaussianBeam(AngledFieldSource, PlanarSource):
+class AstigmaticGaussianBeam(AngledFieldSource, PlanarSource, BroadbandSource):
     """This class implements the simple astigmatic Gaussian beam described in Kochkina et al.,
     Applied Optics, vol. 52, issue 24, 2013. The simple astigmatic Guassian distribution allows
     both an elliptical intensity profile and different waist locations for the two principal axes
@@ -677,4 +823,5 @@ SourceType = Union[
     AstigmaticGaussianBeam,
     ModeSource,
     PlaneWave,
+    CustomFieldSource,
 ]
